import "core-js/modules/es.symbol";
import "core-js/modules/es.symbol.description";
import "core-js/modules/es.symbol.iterator";
import "core-js/modules/es.array.filter";
import "core-js/modules/es.array.for-each";
import "core-js/modules/es.array.is-array";
import "core-js/modules/es.array.iterator";
import "core-js/modules/es.date.to-string";
import "core-js/modules/es.object.define-properties";
import "core-js/modules/es.object.define-property";
import "core-js/modules/es.object.get-own-property-descriptor";
import "core-js/modules/es.object.get-own-property-descriptors";
import "core-js/modules/es.object.keys";
import "core-js/modules/es.object.to-string";
import "core-js/modules/es.promise";
import "core-js/modules/es.regexp.to-string";
import "core-js/modules/es.string.iterator";
import "core-js/modules/web.dom-collections.for-each";
import "core-js/modules/web.dom-collections.iterator";
import "regenerator-runtime/runtime";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

import { ref, isRef, watch, onBeforeUnmount } from "@vue/composition-api";

/**
 * Async helper function that returns three reactive values:
 * * `isLoading`, a boolean that is true during pending state;
 * * `data`, contains the resolved value in the fulfilled state; and
 * * `error`, contains the exception in the rejected state.
 *
 * It returns the following functions as well:
 * * `abort`, that aborts the current promise
 * * `retry`, that retries the original promise
 *
 * @param promiseFn (optionally ref to) function that returns a Promise.
 * @param params (optionally ref to) parameters passed as first argument to the promise function.
 * @returns Object literal containing `isLoading`, `error` and `data` value wrappers and `abort` and `retry`
 * functions.
 */
export function useAsync(promiseFn, params) {
  // always wrap arguments
  var wrapPromiseFn = isRef(promiseFn) ? promiseFn : ref(promiseFn);
  var wrapParams = isRef(params) ? params : ref(params); // create empty return values

  var isLoading = ref(false);
  var error = ref();
  var data = ref(); // abort controller

  var controller;

  function abort() {
    isLoading.value = false;

    if (controller !== undefined) {
      controller.abort();
      controller = undefined;
    }
  }

  function retry() {
    // unwrap the original promise as it is optionally wrapped
    var origPromiseFn = wrapPromiseFn.value; // create a new promise and trigger watch

    wrapPromiseFn.value = function _callee(params, signal) {
      return regeneratorRuntime.async(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              return _context.abrupt("return", origPromiseFn(params, signal));

            case 1:
            case "end":
              return _context.stop();
          }
        }
      });
    };
  } // watch for change in arguments, which triggers immediately initially


  var watched = [wrapPromiseFn, wrapParams];
  watch(watched, function _callee2(_ref) {
    var _ref2, newPromiseFn, newParams, result;

    return regeneratorRuntime.async(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _ref2 = _slicedToArray(_ref, 2), newPromiseFn = _ref2[0], newParams = _ref2[1];
            _context2.prev = 1;
            abort();
            isLoading.value = true;
            controller = new AbortController();
            _context2.next = 7;
            return regeneratorRuntime.awrap(newPromiseFn(newParams, controller.signal));

          case 7:
            result = _context2.sent;
            error.value = undefined;
            data.value = result;
            _context2.next = 16;
            break;

          case 12:
            _context2.prev = 12;
            _context2.t0 = _context2["catch"](1);
            error.value = _context2.t0;
            data.value = undefined;

          case 16:
            _context2.prev = 16;
            isLoading.value = false;
            return _context2.finish(16);

          case 19:
          case "end":
            return _context2.stop();
        }
      }
    }, null, null, [[1, 12, 16, 19]]);
  });
  onBeforeUnmount(abort);
  return {
    isLoading: isLoading,
    error: error,
    data: data,
    abort: abort,
    retry: retry
  };
}
/**
 * Fetch helper function that accepts the same arguments as `fetch` and returns the same values as `useAsync`.
 * If the `Accept` header is set to `application/json` in the `requestInit` object, the response will be parsed as JSON,
 * else text.
 *
 * @param requestInfo (optionally ref to) URL or request object.
 * @param requestInit (optionally ref to) init parameters for the request.
 * @returns Object literal containing same return values as `useAsync`.
 */

export function useFetch(requestInfo) {
  var requestInit = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  // always wrap arguments
  var wrapReqInfo = isRef(requestInfo) ? requestInfo : ref(requestInfo);
  var wrapReqInit = isRef(requestInit) ? requestInit : ref(requestInit);

  function doFetch(params, signal) {
    var requestInit, res, headers;
    return regeneratorRuntime.async(function doFetch$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            requestInit = wrapReqInit.value;
            _context3.next = 3;
            return regeneratorRuntime.awrap(fetch(wrapReqInfo.value, _objectSpread({}, requestInit, {
              signal: signal
            })));

          case 3:
            res = _context3.sent;

            if (res.ok) {
              _context3.next = 6;
              break;
            }

            throw res;

          case 6:
            // TODO figure out how to use typed headers
            headers = requestInit.headers;

            if (!(headers && headers.Accept === "application/json")) {
              _context3.next = 9;
              break;
            }

            return _context3.abrupt("return", res.json());

          case 9:
            return _context3.abrupt("return", res.text());

          case 10:
          case "end":
            return _context3.stop();
        }
      }
    });
  } // wrap original fetch function in value


  var wrapPromiseFn = ref(doFetch); // watch for change in arguments, which triggers immediately initially

  watch([wrapReqInfo, wrapReqInit], function _callee4() {
    return regeneratorRuntime.async(function _callee4$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            // create a new promise and trigger watch
            wrapPromiseFn.value = function _callee3(params, signal) {
              return regeneratorRuntime.async(function _callee3$(_context4) {
                while (1) {
                  switch (_context4.prev = _context4.next) {
                    case 0:
                      return _context4.abrupt("return", doFetch(params, signal));

                    case 1:
                    case "end":
                      return _context4.stop();
                  }
                }
              });
            };

          case 1:
          case "end":
            return _context5.stop();
        }
      }
    });
  });
  return useAsync(wrapPromiseFn);
}
//# sourceMappingURL=index.js.map